/*
 * dac.c part of XMEGA Oscilloscope
 *
 * Created: 2016-12-09 22:22:51
 * Atmel Studio 6 (Version: 6.2.1563 - Service Pack 2)
 *  Author: Krzysztof Markiewicz
 *  www.obbo.pl
 *
 * This program is distributed under the terms of the GNU General Public License
 */ 

#include <avr/interrupt.h>
#include <util/delay.h>
#include "dac.h"
#include "calibration.h"


uint8_t EEMEM dac_eegain = 0x80;
uint8_t EEMEM dac_eeoffset = 0x80;


uint8_t dac_CalibrationTune(uint8_t val, const int8_t step);


void dac_Init(void) {
	DACB.CTRLC = DAC_REFSEL_AREFB_gc;							
	DACB.EVCTRL = DAC_EVSEL_3_gc;			
	DACB.CTRLB = DAC_CHSEL_SINGLE_gc;						
	DACB.CH0OFFSETCAL = eeprom_read_byte(&dac_eeoffset);		
	DACB.CH0GAINCAL = eeprom_read_byte(&dac_eegain);
	DACB.CTRLA = DAC_CH0EN_bm | DAC_ENABLE_bm;
}

void dac_DMAStart(uint16_t *buffer, uint16_t length) {
	if (DMA.CH3.CTRLA & DMA_CH_ENABLE_bm) {
		dac_DMAStop();
	}
	DMA.CH3.ADDRCTRL = DMA_CH_SRCRELOAD_BLOCK_gc | DMA_CH_SRCDIR_INC_gc | DMA_CH_DESTRELOAD_BURST_gc | DMA_CH_DESTDIR_INC_gc;
	DMA.CH3.TRFCNT = 2 * length;
	DMA.CH3.SRCADDR0 = (uint16_t)(buffer) & 0xff;
	DMA.CH3.SRCADDR1 = ((uint16_t)(buffer) >> 8) & 0xff;
	DMA.CH3.SRCADDR2 = 0;
	DMA.CH3.DESTADDR0 = ((uint16_t)(&DACB.CH0DATAL)) & 0xff;
	DMA.CH3.DESTADDR1 = (((uint16_t)(&DACB.CH0DATAL)) >> 8) & 0xff;
	DMA.CH3.DESTADDR2 = 0;
	DMA.CH3.CTRLA = DMA_CH_REPEAT_bm | DMA_CH_SINGLE_bm | DMA_CH_BURSTLEN_2BYTE_gc;
	DACB.CTRLB |= DAC_CH0TRIG_bm;
	DMA.CH3.TRIGSRC = DMA_CH_TRIGSRC_DACB_CH0_gc;
	DMA.CH3.CTRLA |= DMA_CH_ENABLE_bm;
}

void dac_DMAStop(void) {
	uint8_t t = 100;
		
	DMA.CH3.CTRLA &= ~(DMA_CH_ENABLE_bm);
	while ((DMA.CH3.CTRLA & DMA_CH_ENABLE_bm) && (t != 0)) {
		_delay_us(1);
		t--;
	}
	DMA.CH3.TRIGSRC = DMA_CH_TRIGSRC_OFF_gc;
	DMA.CH3.CTRLA |= DMA_CH_RESET_bm;
	DACB.CTRLB &= ~(DAC_CH0TRIG_bm);
	TCD1.CTRLA = 0;
}

void dac_BufferStop(void) {
	dac_Off();
}

void dac_On(void) {
	DACB.CH0DATA = DAC_OUT_Zero;
}

void dac_Off(void) {    
	dac_DMAStop();
	DACB.CH0DATA = DAC_OUT_Zero;
	DACB.CH0DATA = DAC_OUT_Zero;
}

void dac_Calibrate(const uint8_t type, const int8_t step) {
	uint8_t tmpoffset, tmpgain; 
	
	tmpoffset = DACB.CH0OFFSETCAL;
	tmpgain = DACB.CH0GAINCAL;
	if (type == CALIBRATION_DAC_Offset) {
		tmpoffset = dac_CalibrationTune(tmpoffset, step);
		DACB.CH0OFFSETCAL = tmpoffset;
		if (step == 0) eeprom_write_byte(&dac_eeoffset, tmpoffset);
	}
	if (type == CALIBRATION_DAC_Gain) {
		tmpgain = dac_CalibrationTune(tmpgain, step);
		DACB.CH0GAINCAL = tmpgain;
		if (step == 0) eeprom_write_byte(&dac_eegain, tmpgain);
	}
}

uint8_t dac_CalibrationTune(uint8_t val, const int8_t step) {   
	if (val < 128) val = 127 - val;
	if (((step > 0) && (val > 0)) || ((step < 0) && (val < 255))) val -= step;
	if (val < 128) val = 127 - val;
	return val;
}