/*
 * usb.c part of XMEGA Oscilloscope
 *
 * Created: 2016-12-16
 * Atmel Studio 6 (Version: 6.2.1563 - Service Pack 2)
 *  Author: Krzysztof Markiewicz
 *  www.obbo.pl
 *
 * This program is distributed under the terms of the GNU General Public License
 */ 

#include "usb.h"
#include <util/delay.h>



// Send data buffer using bulk method
uint8_t usb_SendDataBuffer(uint8_t *buffer, uint16_t length) {
	uint16_t sended = 0;
	uint16_t t = USB_COMUNICATION_TIMEOUT_ms;
	uint8_t result = 0;
	
	do {
		if ((endpoints[1].in.STATUS & USB_EP_TRNCOMPL0_bm)) {
			endpoints[1].in.AUXDATA = 0;
			endpoints[1].in.DATAPTR = (uint16_t)buffer + sended;
			if ((length - sended) > USB_MAX_BYTES) {
				endpoints[1].in.CNT = USB_MAX_BYTES;
				sended += USB_MAX_BYTES;
			} else {
				endpoints[1].in.CNT = length-sended;
				sended = length;
			}
			endpoints[1].in.STATUS &= ~(USB_EP_TRNCOMPL0_bm | USB_EP_BUSNACK0_bm | USB_EP_OVF_bm);
			t = USB_COMUNICATION_TIMEOUT_ms;
		}
		if (!testbit(GPIO_Status, MAIN_STATUS_USB_Connected)) {
			return result;
		}
		_delay_ms(1);
		t--;
	} while ((sended < length) && (t != 0));
	if (t == 0) {
		setbit(result, USB_ERROR_Timeout);
	}
	return result;	
}

