; XMEGA Oscilloscope and Development Kit
;
; Gabotronics
; March 2012
;
; Copyright 2012 Gabriel Anzziani
;
; This program is distributed under the terms of the GNU General Public License
;
; www.gabotronics.com
; email me at: gabriel@gabotronics.com
;
;
; Last modified:
; Atmel Studio 6 (Version: 6.2.1563 - Service Pack 2)
;  2016-2017 Krzysztof Markiewicz
;  www.obbo.pl
;

#include <avr/io.h>


; Digital Trigger Rising Edge
;----------------------------------------------------------------------------;
;
.global trigupCHD
.func trigupCHD
trigupCHD:
    ldi ZL, lo8(pm(tableup))
    ldi ZH, hi8(pm(tableup))
    add ZL, r24
    adc ZH, r1
    ijmp
tableup:
    rjmp trigpc0u
    rjmp trigpc1u
    rjmp trigpc2u
    rjmp trigpc3u
    rjmp trigpc4u
    rjmp trigpc5u
    rjmp trigpc6u
    rjmp trigpc7u
    rjmp trigpd1u
	rjmp trigpdsu
.endfunc

; trigger on PC0
;----------------------------------------------------------------------------;
;
.global trigpc0u
.func trigpc0u
trigpc0u:
    sbic    0x000d, 0        ; 2 cycle ; Status update? (GPIOD)
    ret
    sbic    0x001a, 0        ; Check if bit VPORT2.0 is low
    rjmp    trigpc0u
0:
    sbic    0x000d, 0        ; 2 cycle ; Status update? (GPIOD)
    ret
    sbis    0x001a, 0        ; Check if bit VPORT2.0 is high
    rjmp    0b
    rjmp    post
.endfunc

; trigger on PC1
;----------------------------------------------------------------------------;
;
.global trigpc1u
.func trigpc1u
trigpc1u:
    sbic    0x000d, 0        ; 2 cycle ; Status update? (GPIOD)
    ret
    sbic    0x001a, 1        ; Check if bit VPORT2.1 is low
    rjmp    trigpc1u
0:
    sbic    0x000d, 0        ; 2 cycle ; Status update? (GPIOD)
    ret
    sbis    0x001a, 1        ; Check if bit VPORT2.1 is high
    rjmp    0b
    rjmp    post
.endfunc

; trigger on PC2
;----------------------------------------------------------------------------;
;
.global trigpc2u
.func trigpc2u
trigpc2u:
    sbic    0x000d, 0        ; 2 cycle ; Status update? (GPIOD)
    ret
    sbic    0x001a, 2        ; Check if bit VPORT2.1 is low
    rjmp    trigpc2u
0:
    sbic    0x000d, 0        ; 2 cycle ; Status update? (GPIOD)
    ret
    sbis    0x001a, 2        ; Check if bit VPORT2.1 is high
    rjmp    0b
    rjmp    post
.endfunc

; trigger on PC3
;----------------------------------------------------------------------------;
;
.global trigpc3u
.func trigpc3u
trigpc3u:
    sbic    0x000d, 0        ; 2 cycle ; Status update? (GPIOD)
    ret
    sbic    0x001a, 3        ; Check if bit VPORT2.1 is low
    rjmp    trigpc3u
0:
    sbic    0x000d, 0        ; 2 cycle ; Status update? (GPIOD)
    ret
    sbis    0x001a, 3        ; Check if bit VPORT2.1 is high
    rjmp    0b
    rjmp    post
.endfunc


; trigger on PC4
;----------------------------------------------------------------------------;
;
.global trigpc4u
.func trigpc4u
trigpc4u:
    sbic    0x000d, 0        ; 2 cycle ; Status update? (GPIOD)
    ret
    sbic    0x001a, 4        ; Check if bit VPORT2.1 is low
    rjmp    trigpc4u
0:
    sbic    0x000d, 0        ; 2 cycle ; Status update? (GPIOD)
    ret
    sbis    0x001a, 4        ; Check if bit VPORT2.1 is high
    rjmp    0b
    rjmp    post
.endfunc


; trigger on PC5
;----------------------------------------------------------------------------;
;
.global trigpc5u
.func trigpc5u
trigpc5u:
    sbic    0x000d, 0        ; 2 cycle ; Status update? (GPIOD)
    ret
    sbic    0x001a, 5        ; Check if bit VPORT2.1 is low
    rjmp    trigpc5u
0:
    sbic    0x000d, 0        ; 2 cycle ; Status update? (GPIOD)
    ret
    sbis    0x001a, 5        ; Check if bit VPORT2.1 is high
    rjmp    0b
    rjmp    post
.endfunc


; trigger on PC6
;----------------------------------------------------------------------------;
;
.global trigpc6u
.func trigpc6u
trigpc6u:
    sbic    0x000d, 0        ; 2 cycle ; Status update? (GPIOD)
    ret
    sbic    0x001a, 6        ; Check if bit VPORT2.1 is low
    rjmp    trigpc6u
0:
    sbic    0x000d, 0        ; 2 cycle ; Status update? (GPIOD)
    ret
    sbis    0x001a, 6        ; Check if bit VPORT2.1 is high
    rjmp    0b
    rjmp    post
.endfunc


; trigger on PC7
;----------------------------------------------------------------------------;
;
.global trigpc7u
.func trigpc7u
trigpc7u:
    sbic    0x000d, 0        ; 2 cycle ; Status update? (GPIOD)
    ret
    sbic    0x001a, 7        ; Check if bit VPORT2.1 is low
    rjmp    trigpc7u
0:
    sbic    0x000d, 0        ; 2 cycle ; Status update? (GPIOD)
    ret
    sbis    0x001a, 7        ; Check if bit VPORT2.1 is high
    rjmp    0b
    rjmp    post
.endfunc

; trigger on PD1 (external trigger)
;----------------------------------------------------------------------------;
;
.global trigpd1u
.func trigpd1u
trigpd1u:
    sbic    0x000d, 0        ; 2 cycle ; Status update? (GPIOD)
    ret
    sbic    0x001e, 1        ; Check if bit VPORT3.1 is low
    rjmp    trigpd1u
0:
    sbic    0x000d, 0        ; 2 cycle ; Status update? (GPIOD)
    ret
    sbis    0x001e, 1        ; Check if bit VPORT3.1 is high
    rjmp    0b
    rjmp    post
.endfunc

; trigger on PD0 (external trigger with schmitt input)
;----------------------------------------------------------------------------;
;
.global trigpdsu
.func trigpdsu
trigpdsu:
    sbic    0x000d, 0        ; 2 cycle ; Status update? (GPIOD)
    ret
    sbic    0x001e, 0        ; Check if bit VPORT3.0 is low
    rjmp    trigpdsu
0:
    sbic    0x000d, 0        ; 2 cycle ; Status update? (GPIOD)
    ret
    sbis    0x001e, 0        ; Check if bit VPORT3.0 is high
    rjmp    0b
    rjmp    post
.endfunc

; Digital Trigger Rising Edge
;----------------------------------------------------------------------------;
;
.global trigdownCHD
.func trigdownCHD
trigdownCHD:
    ldi ZL, lo8(pm(tabledown))
    ldi ZH, hi8(pm(tabledown))
    add ZL, r24
    adc ZH, r1
    ijmp
tabledown:
    rjmp trigpc0d
    rjmp trigpc1d
    rjmp trigpc2d
    rjmp trigpc3d
    rjmp trigpc4d
    rjmp trigpc5d
    rjmp trigpc6d
    rjmp trigpc7d
    rjmp trigpd1d
	rjmp trigpdsd
.endfunc

; trigger on PC0
;----------------------------------------------------------------------------;
;
.global trigpc0d
.func trigpc0d
trigpc0d:
    sbic    0x000d, 0        ; 2 cycle ; Status update? (GPIOD)
    ret
    sbis    0x001a, 0        ; Check if bit VPORT2.0 is high
    rjmp    trigpc0d
0:
    sbic    0x000d, 0        ; 2 cycle ; Status update? (GPIOD)
    ret
    sbic    0x001a, 0        ; Check if bit VPORT2.0 is low
    rjmp    0b
    rjmp    post
.endfunc

; trigger on PC1
;----------------------------------------------------------------------------;
;
.global trigpc1d
.func trigpc1d
trigpc1d:
    sbic    0x000d, 0        ; 2 cycle ; Status update? (GPIOD)
    ret
    sbis    0x001a, 1        ; Check if bit VPORT2.1 is high
    rjmp    trigpc1d
0:
    sbic    0x000d, 0        ; 2 cycle ; Status update? (GPIOD)
    ret
    sbic    0x001a, 1        ; Check if bit VPORT2.1 is low
    rjmp    0b
    rjmp    post
.endfunc

; trigger on PC2
;----------------------------------------------------------------------------;
;
.global trigpc2d
.func trigpc2d
trigpc2d:
    sbic    0x000d, 0        ; 2 cycle ; Status update? (GPIOD)
    ret
    sbis    0x001a, 2        ; Check if bit VPORT2.1 is high
    rjmp    trigpc2d
0:
    sbic    0x000d, 0        ; 2 cycle ; Status update? (GPIOD)
    ret
    sbic    0x001a, 2        ; Check if bit VPORT2.1 is low
    rjmp    0b
    rjmp    post
.endfunc


; trigger on PC3
;----------------------------------------------------------------------------;
;
.global trigpc3d
.func trigpc3d
trigpc3d:
    sbic    0x000d, 0        ; 2 cycle ; Status update? (GPIOD)
    ret
    sbis    0x001a, 3        ; Check if bit VPORT2.1 is high
    rjmp    trigpc3d
0:
    sbic    0x000d, 0        ; 2 cycle ; Status update? (GPIOD)
    ret
    sbic    0x001a, 3        ; Check if bit VPORT2.1 is low
    rjmp    0b
    rjmp    post
.endfunc


; trigger on PC4
;----------------------------------------------------------------------------;
;
.global trigpc4d
.func trigpc4d
trigpc4d:
    sbic    0x000d, 0        ; 2 cycle ; Status update? (GPIOD)
    ret
    sbis    0x001a, 4        ; Check if bit VPORT2.1 is high
    rjmp    trigpc4d
0:
    sbic    0x000d, 0        ; 2 cycle ; Status update? (GPIOD)
    ret
    sbic    0x001a, 4        ; Check if bit VPORT2.1 is low
    rjmp    0b
    rjmp    post
.endfunc


; trigger on PC5
;----------------------------------------------------------------------------;
;
.global trigpc5d
.func trigpc5d
trigpc5d:
    sbic    0x000d, 0        ; 2 cycle ; Status update? (GPIOD)
    ret
    sbis    0x001a, 5        ; Check if bit VPORT2.1 is high
    rjmp    trigpc5d
0:
    sbic    0x000d, 0        ; 2 cycle ; Status update? (GPIOD)
    ret
    sbic    0x001a, 5        ; Check if bit VPORT2.1 is low
    rjmp    0b
    rjmp    post
.endfunc


; trigger on PC6
;----------------------------------------------------------------------------;
;
.global trigpc6d
.func trigpc6d
trigpc6d:
    sbic    0x000d, 0        ; 2 cycle ; Status update? (GPIOD)
    ret
    sbis    0x001a, 6        ; Check if bit VPORT2.1 is high
    rjmp    trigpc6d
0:
    sbic    0x000d, 0        ; 2 cycle ; Status update? (GPIOD)
    ret
    sbic    0x001a, 6        ; Check if bit VPORT2.1 is low
    rjmp    0b
    rjmp    post
.endfunc


; trigger on PC7
;----------------------------------------------------------------------------;
;
.global trigpc7d
.func trigpc7d
trigpc7d:
    sbic    0x000d, 0        ; 2 cycle ; Status update? (GPIOD)
    ret
    sbis    0x001a, 7        ; Check if bit VPORT2.1 is high
    rjmp    trigpc7d
0:
    sbic    0x000d, 0        ; 2 cycle ; Status update? (GPIOD)
    ret
    sbic    0x001a, 7        ; Check if bit VPORT2.1 is low
    rjmp    0b
    rjmp    post
.endfunc

; trigger on PD1 (external trigger)
;----------------------------------------------------------------------------;
;
.global trigpd1d
.func trigpd1d
trigpd1d:
    sbic    0x000d, 0        ; 2 cycle ; Status update? (GPIOD)
    ret
    sbis    0x001e, 1        ; Check if bit VPORT3.1 is high
    rjmp    trigpd1d
0:
    sbic    0x000d, 0        ; 2 cycle ; Status update? (GPIOD)
    ret
    sbic    0x001e, 1        ; Check if bit VPORT3.1 is low
    rjmp    0b
    rjmp    post
.endfunc

; trigger on PD0 (external trigger with schmitt input)
;----------------------------------------------------------------------------;
;
.global trigpdsd
.func trigpdsd
trigpdsd:
    sbic    0x000d, 0        ; 2 cycle ; Status update? (GPIOD)
    ret
    sbis    0x001e, 0        ; Check if bit VPORT3.0 is high
    rjmp    trigpdsd
0:
    sbic    0x000d, 0        ; 2 cycle ; Status update? (GPIOD)
    ret
    sbic    0x001e, 0        ; Check if bit VPORT3.0 is low
    rjmp    0b
    rjmp    post
.endfunc

; falling edge trigger detect on CH1
;----------------------------------------------------------------------------;
;                 data1  ___.
;                            \
;  triggervalue - - - - - - - \
;                              \.___ data2
;
.global trigdownCH1
.func trigdownCH1
trigdownCH1:
    subi    r24, 0x02       ; r24 = (signed)(tlevelo - 2)
0:
    lds     r0, 0x0224      ; 2 cycle : r21 = ADCA CH0.RESL
    cp      r24, r0         ; 1 cycle ; compare triggervalue with data1
    brge    1f              ; 1 cycle ; next step if tlevel >= data1
    sbis    0x000d, 0       ; 1 cycle ; Status update? (GPIOD)
    rjmp    0b              ; 2 cycles
                            ; 17 cycles
    ret
1:
    subi    r24, 0xfc       ; r24 = (signed)(tlevelo + 2)
2:
    lds     r0, 0x0224      ; 2 cycle : r0 = ADCA CH0.RESL
    cp      r24, r0         ; 1 cycle ; compare triggervalue with data2
    brlt    post            ; 1 cycle ; finish if tlevelo < data2
    sbis    0x000d, 0       ; 1 cycle ; Status update? (GPIOD)
    rjmp    2b              ; 2 cycles
                            ; 7 cycles
    ret
.endfunc

; falling edge trigger detect on CH2
;----------------------------------------------------------------------------;
;                 data1  ___.
;                            \
;  triggervalue - - - - - - - \
;                              \.___ data2
;
.global trigdownCH2
.func trigdownCH2
trigdownCH2:
    subi    r24, 0x02       ; r24 = (signed)(tlevelo - 2)
0:
    lds     r0, 0x0264      ; 2 cycle : r0 = ADCA CH1.RESL
    cp      r24, r0         ; 1 cycle ; compare triggervalue with data1
    brge    1f              ; 1 cycle ; next step if tlevelo >= data1
    sbis    0x000d, 0       ; 1 cycle ; Status update? (GPIOD)
    rjmp    0b              ; 2 cycles
                            ; 7 cycles
    ret
1:
    subi    r24, 0xfc       ; r24 = (signed)(tlevelo + 2)
2:
    lds     r0, 0x0264      ; 2 cycle : r0 = ADCA CH1.RESL
    cp      r24, r0         ; 1 cycle ; compare triggervalue with data2
    brlt    post            ; 1 cycle ; finish if tlevelo < data2
    sbis    0x000d, 0       ; 1 cycle ; Status update? (GPIOD)   
    rjmp    2b              ; 2 cycles
                            ; 7 cycles
    ret
.endfunc

; rising edge trigger detect on CH1
;----------------------------------------------------------------------------;
;                              .___ data2
;                              /
;  triggervalue - - - - - - - /
;                  data1 ___./
;
.global trigupCH1
.func trigupCH1
trigupCH1:
    subi    r24, 0xfe       ; r24 = (signed)(tlevelo + 2)                         
0:
    lds     r0, 0x0224      ; 2 cycle : r0 = ADCA CH0.RESL
    cp      r24, r0         ; 1 cycle ; compare triggervalue with data1
    brlt    1f              ; 1 cycle ; next step if tlevelo < data1
    sbis    0x000d, 0       ; 1 cycle ; Status update? (GPIOD)
    rjmp    0b              ; 2 cycles
                            ; 7 cycles
    ret
1:
    subi    r24, 0x02       ; r24 = (signed)(tlevelo - 2)
2:
    lds     r0, 0x0224      ; 2 cycle : r0 = ADCA CH0.RESL
    cp      r24, r0         ; 1 cycle ; compare triggervalue with data2
    brge    post            ; 1 cycle ; finish if tlevelo >= data2
    sbis    0x000d, 0       ; 1 cycle ; Status update? (GPIOD)
    rjmp    2b              ; 2 cycles
                            ; 7 cycles
    ret
.endfunc

; rising edge trigger detect on CH2
;----------------------------------------------------------------------------;
;                              .___ data2
;                              /
;  triggervalue - - - - - - - /
;                  data1 ___./
;
.global trigupCH2
.func trigupCH2
trigupCH2:
    subi    r24, 0xfe       ; r24 = (signed)(tlevelo + 2)
0:
    lds     r0, 0x0264      ; 2 cycle : r0 = ADCA CH1.RESL
    cp      r24, r0         ; 1 cycle ; compare triggervalue with data1
    brlt    1f              ; 1 cycle ; next step if tlevelo < data1
    sbis    0x000d, 0       ; 1 cycle ; Status update? (GPIOD)  
    rjmp    0b              ; 2 cycles
                            ; 7 cycles
    ret
1:
    subi    r24, 0x02       ; r24 = (signed)(tlevelo - 2)
2:
    lds     r0, 0x0264      ; 2 cycle : r0 = ADCA CH1.RESL
    cp      r24, r0         ; 1 cycle ; compare triggervalue with data1
    brge    post            ; 1 cycle ; finish if tlevelo >= data2
    sbis    0x000d, 0       ; 1 cycle ; Status update? (GPIOD)   
    rjmp    2b              ; 2 cycles
                            ; 7 cycles
    ret
.endfunc

; Post trigger
;----------------------------------------------------------------------------;
;
.global post
.func post
post:     
    ; Start TCC1 and count remaining samples
    LDI     r24, 0x09       ; Event EVSYS.CH1
    STS     0x0840, r24     ; Store 0x09 in TCC1.CTRLA (count EVSYS.CH1 event)
    sbi     0x000c, 0       ; Set trigger (GPIOC)
    in      r24, 0x0000     ; load srate (GPIO0) in r24
    cpi     r24, 11         ; compare srate with 11
    brcc    1f              ; exit if srate>=11
    LDI     r24, 0x14       ; Prepare R24: signed mode, NO free run, 8 bit
0:
    lds     r0, 0x084c      ; TCC1.INTFLAGS
    sbrs    r0, 0           ; jump if overflow
    rjmp    0b
    STS     0x0a00, r1      ; Stop Timer TCE0 (for srate > 5)
	STS		0x0180, r1      ; Stop EVSYS.CH0
    STS     0x0201, r24     ; Store 0x14 in ADCA.CTRLB (signed mode, NO free run, 8 bit)
	STS     0x0241, r24     ; Store 0x14 in ADCB.CTRLB (signed mode, NO free run, 8 bit)
1:
    ret
.endfunc

; window trigger detect on CH1
;----------------------------------------------------------------------------;
; r22 contains window1, r24 contains window2
.global windowCH1
.func windowCH1
windowCH1:
	subi    r24, 0x02		; r24 = (signed)(tlevelo - 2)
	subi    r22, 0xfe		; r22 = (signed)(tlevelo + 2)
0:
    lds     r0, 0x0224      ; 2 cycle : r21 = ADCA CH0.RESL
    cp      r24, r0         ; 1 cycle ; compare window1 with data
	sbis    0x000d, 0       ; 1 cycle ; Status update? (GPIOD)
    brlt    0b              ; 1 cycle ; continue if window1 >= data
    cp      r0, r22         ; 1 cycle ; compare window2 with data
	sbis    0x000d, 0       ; 1 cycle ; Status update? (GPIOD)
    brlt    0b              ; 1 cycle ; continue if window2 < data
	subi    r24, 0xfc		; r24 = (signed)(tlevelo + 2)
	subi    r22, 0x04		; r22 = (signed)(tlevelo - 2)
1:
    lds     r0, 0x0224      ; 2 cycle : r21 = ADCA CH0.RESL
    cp      r24, r0         ; 1 cycle ; compare window1 with data
    brlt    post            ; 1 cycle ; continue if window1 >= data
    cp      r0, r22         ; 1 cycle ; compare window2 with data
    brlt    post            ; 1 cycle ; continue if window2 < data
    sbis    0x000d, 0       ; 1 cycle ; Status update? (GPIOD)
    rjmp    1b              ; 2 cycles
    ret
.endfunc

; window trigger detect on CH2
;----------------------------------------------------------------------------;
; r22 contains window1, r24 contains window2
.global windowCH2
.func windowCH2
windowCH2:
	subi    r24, 0x02		; r24 = (signed)(tlevelo - 2)
	subi    r22, 0xfe		; r24 = (signed)(tlevelo + 2)
0:
    lds     r0, 0x0264      ; 2 cycle : r21 = ADCA CH0.RESL
    cp      r24, r0         ; 1 cycle ; compare window1 with data
	sbis    0x000d, 0       ; 1 cycle ; Status update? (GPIOD)
    brlt    0b              ; 1 cycle ; continue if window1 >= data
    cp      r0, r22         ; 1 cycle ; compare window2 with data
	sbis    0x000d, 0       ; 1 cycle ; Status update? (GPIOD)
    brlt    0b              ; 1 cycle ; continue if window2 < data
	subi    r24, 0xfc		; r24 = (signed)(tlevelo + 2)
	subi    r22, 0x04		; r24 = (signed)(tlevelo - 2)
1:
    lds     r0, 0x0264      ; 2 cycle : r21 = ADCA CH0.RESL
    cp      r24, r0         ; 1 cycle ; compare window1 with data
    brlt    post            ; 1 cycle ; continue if window1 >= data
    cp      r0, r22         ; 1 cycle ; compare window2 with data
    brlt    post            ; 1 cycle ; continue if window2 < data
    sbis    0x000d, 0       ; 1 cycle ; Status update? (GPIOD)
    rjmp    1b              ; 2 cycles
    ret
.endfunc

; negative slope trigger detect on CH1
;----------------------------------------------------------------------------;
;                    old ___.
;                            \
;                           ||\
;                           >< \.___ new
;                    triggervalue
.global slopedownCH1
.func slopedownCH1
slopedownCH1:
    push    r23
    lds     r0, 0x0224      ; 2 cycle : r0 = ADCA CH0.RESL old value
0:
    lds     r23, 0x0224     ; 2 cycle : r23 = ADCA CH0.RESL
    cp      r0, r23         ; 1 cycle ; determine difference
    mov     r0, r23         ; 1 cycle ; copy new to old
    brlt    2f              ; next if old < new
    sbis    0x000d, 0       ; 1 cycle ; Status update? (GPIOD)
    rjmp    0b              ; 2 cycles
                            ; 17 cycles
    pop     r23
    ret
2:
    lds     r23, 0x0224     ; 2 cycle : r23 = ADCA CH0.RESL
    sub     r23, r0         ; 1 cycle ; determine difference
    cp      r23, r24        ; 1 cycle ; compare with triggervalue
    mov     r0, r23         ; 1 cycle ; copy new to old
    brge    3f              ; 1 cycle ; tlevelo < data ?
    sbis    0x000d, 0       ; 1 cycle ; Status update? (GPIOD)
    rjmp    2b              ; 2 cycles
    pop     r23
    ret
3:  pop     r23
    rjmp    post
.endfunc 

; negative slope trigger detect on CH2
;----------------------------------------------------------------------------;
;                    old ___.
;                            \
;                           ||\
;                           >< \.___ new
;                    triggervalue
.global slopedownCH2
.func slopedownCH2
slopedownCH2:
    push    r23
    lds     r0, 0x0264      ; 2 cycle : r0 = ADCA CH1.RESL old value
0:
    lds     r23, 0x0264     ; 2 cycle : r23 = ADCA CH1.RESL
    cp      r0, r23         ; 1 cycle ; determine difference
    mov     r0, r23         ; 1 cycle ; copy new to old
    brlt    2f              ; next if old + 3 < new
    sbis    0x000d, 0       ; 1 cycle ; Status update? (GPIOD)  
    rjmp    0b              ; 2 cycles
                            ; 17 cycles
    pop     r23
    ret
2:
    lds     r23, 0x0264     ; 2 cycle : r23 = ADCA CH1.RESL
    sub     r23, r0         ; 1 cycle ; determine difference
    cp      r23, r24        ; 1 cycle ; compare with triggervalue
    mov     r0, r23         ; 1 cycle ; copy new to old
    brge    3f              ; 1 cycle ; tlevelo < data ?
    sbis    0x000d, 0       ; 1 cycle ; Status update? (GPIOD)  
    rjmp    2b              ; 2 cycles
    pop     r23
    ret
3:  pop     r23
    rjmp    post
.endfunc

; positive slope trigger detect on CH1
;----------------------------------------------------------------------------;
;                              .___ new
;                              /
;                             /||
;                    old ___./ ><
;                           triggervalue
.global slopeupCH1
.func slopeupCH1
slopeupCH1:
    push    r23
    lds     r0, 0x0224      ; 2 cycle : r0 = ADCA CH0.RESL old value
0:
    lds     r23, 0x0224     ; 2 cycle : r1 = ADCA CH0.RESL
    cp      r23, r0         ; 1 cycle ; determine difference
    mov     r0, r23         ; 1 cycle ; copy new to old
    brlt    2f              ; next if old > new
    sbis    0x000d, 0       ; 1 cycle ; Status update? (GPIOD)  
    rjmp    0b              ; 2 cycles
                            ; 17 cycles
    pop     r23
    ret
2:
    lds     r23, 0x0224     ; 2 cycle : r1 = ADCA CH0.RESL
    sub     r0, r23         ; 1 cycle ; determine difference
    cp      r0, r24         ; 1 cycle ; compare with triggervalue
    mov     r0, r23         ; 1 cycle ; copy new to old
    brge    3f              ; 1 cycle ; tlevelo < data ?
    sbis    0x000d, 0       ; 1 cycle ; Status update? (GPIOD)  
    rjmp    2b              ; 2 cycles
    pop     r23
    ret
3:  pop     r23
    rjmp    post
.endfunc

; positive slope trigger detect on CH2
;----------------------------------------------------------------------------;
;                              .___ new
;                              /
;                             /||
;                    old ___./ ><
;                           triggervalue
.global slopeupCH2
.func slopeupCH2
slopeupCH2:
    push    r23
    lds     r0, 0x0264      ; 2 cycle : r0 = ADCA CH1.RESL old value
0:
    lds     r23, 0x0264     ; 2 cycle : r23 = ADCA CH1.RESL
    cp      r23, r0         ; 1 cycle ; determine difference
    mov     r0, r23         ; 1 cycle ; copy new to old
    brlt    2f              ; next if old > new
    sbis    0x000d, 0       ; 1 cycle ; Status update? (GPIOD)   
    rjmp    0b              ; 2 cycles
                            ; 17 cycles
    pop     r23
    ret
2:
    lds     r23, 0x0264     ; 2 cycle : r23 = ADCA CH1.RESL
    sub     r0, r23         ; 1 cycle ; determine difference
    cp      r0, r24         ; 1 cycle ; compare with triggervalue
    mov     r0, r23         ; 1 cycle ; copy new to old
    brge    3f              ; 1 cycle ; tlevelo < data ?
    sbis    0x000d, 0       ; 1 cycle ; Status update? (GPIOD)  
    rjmp    2b              ; 2 cycles
    pop     r23
    ret
3:  pop     r23
    rjmp    post
	ret
.endfunc


.section .bss
.global endpoints			; Endpoints configuration tables
.align 2					; Align to word
endpoints:  .dc.b 1,2,3,4,5,6,7,8    ; ep0 out
            .dc.b 1,2,3,4,5,6,7,8    ; ep0 in
            .dc.b 1,2,3,4,5,6,7,8    ; ep1 out
            .dc.b 1,2,3,4,5,6,7,8    ; ep1 in