/*
 * awg.h part of XMEGA Oscilloscope
 *
 * Created: 2016-12-11
 * Atmel Studio 6 (Version: 6.2.1563 - Service Pack 2)
 *  Author: Krzysztof Markiewicz
 *  www.obbo.pl
 *
 * This program is distributed under the terms of the GNU General Public License
 */

#ifndef AWG_H_
#define AWG_H_

#include <stdbool.h>
#include <avr/eeprom.h>


extern uint16_t EEMEM awg_wavecustom[];

// export definition to Xlab oscyloskopx.h
#define AWG_WAVE_MIN_Type			0
#define AWG_WAVE_MIN_Duty			1
#define AWG_WAVE_MIN_Frequency		48			// 0.48Hz

#define AWG_WAVE_MAX_Duty			255
#define AWG_WAVE_MAX_Frequency		12500000	// 125kHz
#define AWG_WAVE_MAX_Amplitude		232

#define AWG_WAVE_DEF_Amplitude		116
#define AWG_WAVE_DEF_Type			1
#define AWG_WAVE_DEF_Duty			128
#define AWG_WAVE_DEF_Offset			0
#define AWG_WAVE_DEF_Frequency		10000		// 100Hz
#define AWG_WAVE_DEF_ConstValue		0

#define AWG_SWEEP_SPEED_MAX			127

enum awg_wavetype {
	AWG_WAVE_Sine,
	AWG_WAVE_Square,
	AWG_WAVE_Triangle,
	AWG_WAVE_Exponential,
	AWG_WAVE_Custom,
	AWG_WAVE_Constant,
	AWG_WAVE_WhiteNoise,
	AWG_WAVE_PinkNoise
};

enum awg_sweeptype {
	AWG_SWEEP_TYPE_SweepDown,
	AWG_SWEEP_TYPE_PingPong,
	AWG_SWEEP_TYPE_SweepFrequency,
	AWG_SWEEP_TYPE_SweepAmplitude,
	AWG_SWEEP_TYPE_SweepOffset,
	AWG_SWEEP_TYPE_SweepDuty
};

typedef struct {
	uint8_t     amplitude;
	uint8_t     type;
	uint8_t     duty;
	int8_t      offset;
	uint32_t    frequency;
	int16_t		constvalue;
} awgwave_t;

typedef struct {
	uint8_t		type;
	uint8_t     start;
	uint8_t     end;
	uint8_t     speed;
} awgsweep_t;
// end export definition to Xlab oscyloskopx.h

typedef struct {
	uint16_t *dac;
	uint16_t *wave;
	uint16_t *out;
} awgbuffer_t;


void awg_FillBuffer(awgbuffer_t buffer, uint16_t length, awgsweep_t *sweep, awgwave_t *wave);
void awg_WaveTemplate(uint16_t *buffer, uint16_t length, const uint8_t type);
void awg_WaveConstant(const int16_t val);
void awg_WaveWhiteNoise(awgwave_t *wave, uint8_t *status);
void awg_WavePinkNoise(awgwave_t *wave, uint8_t *status);
void awg_Sweep(awgsweep_t *sweep);
bool awg_SweepEnabled(uint8_t sweeptype);
uint32_t awg_OutputFrequency(uint16_t bufferlength, uint32_t f);


#endif /* AWG_H_ */
