/*
 * calibration.c part of XMEGA Oscilloscope
 *
 * Created: 2017-01-18 23:07:49
 * Atmel Studio 6 (Version: 6.2.1563 - Service Pack 2)
 *  Author: Krzysztof Markiewicz
 *  www.obbo.pl
 *
 * This program is distributed under the terms of the GNU General Public License
 */

#include <util/delay.h>
#include "calibration.h"
#include "multimeter.h"
#include "input.h"
#include "bitlib.h"

int32_t EEMEM adc_offset12CH1[ADC_OFFSET_TAB_LENGTH] = { 0 };
int32_t EEMEM adc_offset12CH2[ADC_OFFSET_TAB_LENGTH] = { 0 };
int8_t EEMEM adc_offset8CH1[ADC_OFFSET_TAB_LENGTH] = { 0 };
int8_t EEMEM adc_offset8CH2[ADC_OFFSET_TAB_LENGTH] = { 0 };


uint8_t calibration_ADCOffset(void) {
	uint8_t result = 0;
	
	result |= (calibration_ADCOffset8bit());
	result |= (calibration_ADCOffset12bit());
	if (result == 0) {
		eeprom_write_byte(&eecalibrated, 1);
		setbit(GPIO_Status, MAIN_STATUS_Calibrated);
		LED_RED_OFF();
	} else {
		eeprom_write_byte(&eecalibrated, 0);
		clrbit(GPIO_Status, MAIN_STATUS_Calibrated);
	}
	return result;
}

void calibration_ADCOffsetClear(uint8_t offset8length, uint8_t offset12length) {
	uint8_t i;
	
	for (i = 0; i < offset8length; i++) {
		eeprom_write_byte((uint8_t *)&adc_offset8CH1[i], 0);
		eeprom_write_byte((uint8_t *)&adc_offset8CH2[i], 0);
	}
	for (i = 0; i < offset12length; i++) {
		eeprom_write_dword((uint32_t *)&adc_offset12CH1[i], 0);
		eeprom_write_dword((uint32_t *)&adc_offset12CH2[i], 0);
	}
}

uint8_t calibration_ADCOffsetCell(uint8_t chcfg, uint8_t chgain) {
	uint8_t result;
	
	result = chgain;
	if (testbit(chcfg, MAIN_CHANNEL_CONTROL_FilterDC)) result += MAIN_CHANNEL_GAIN_RangeOut;
	return result;
}

// Offset for 256 measurements
uint8_t calibration_ADCOffset12bit(void) {
	int32_t ch1vdc32, ch2vdc32;
	uint8_t i, result = 0;
	uint8_t chcfg = 0, chgain;
	
	calibration_ADCOffsetClear(0, ADC_OFFSET_TAB_LENGTH);
	for (i = 0; i < ADC_OFFSET_TAB_LENGTH; i++) {
		if (i < MAIN_CHANNEL_GAIN_RangeOut) {
			clrbit(chcfg, MAIN_CHANNEL_CONTROL_FilterDC);
			chgain = i;
		} else {
			setbit(chcfg, MAIN_CHANNEL_CONTROL_FilterDC);
			chgain = i - MAIN_CHANNEL_GAIN_RangeOut;
		}
		input_DCFilter(testbit(chcfg, MAIN_CHANNEL_CONTROL_FilterDC));
		input_Div10(input_ChannelDiv10(chgain));
		_delay_ms(500);
		mm_MeasureSeriesVoltage12bit(chcfg, chgain, &ch1vdc32, chcfg, chgain, &ch2vdc32, true);
		eeprom_write_dword((uint32_t *)&adc_offset12CH1[calibration_ADCOffsetCell(chcfg, chgain)], ch1vdc32);
		eeprom_write_dword((uint32_t *)&adc_offset12CH2[calibration_ADCOffsetCell(chcfg, chgain)], ch2vdc32);
		if ((ch1vdc32 > 262143) || (ch1vdc32 < -262143)) setbit(result, CALIBRATION_ERROR_OutOfRange);
		if ((ch2vdc32 > 262143) || (ch2vdc32 < -262143)) setbit(result, CALIBRATION_ERROR_OutOfRange);
	}
	return result;
}

// Offset for 1 measurement
uint8_t calibration_ADCOffset8bit(void) {
	int16_t ch1vdc16, ch2vdc16;
	uint8_t i, result = 0;
	uint8_t chcfg = 0, chgain;
	
	calibration_ADCOffsetClear(ADC_OFFSET_TAB_LENGTH, 0);
	for (i = 0; i < ADC_OFFSET_TAB_LENGTH; i++) {
		if (i < MAIN_CHANNEL_GAIN_RangeOut) {
			clrbit(chcfg, MAIN_CHANNEL_CONTROL_FilterDC);
			chgain = i;
		} else {
			setbit(chcfg, MAIN_CHANNEL_CONTROL_FilterDC);
			chgain = i - MAIN_CHANNEL_GAIN_RangeOut;
		}
		input_DCFilter(testbit(chcfg, MAIN_CHANNEL_CONTROL_FilterDC));
		input_Div10(input_ChannelDiv10(chgain));
		_delay_ms(500);
		mm_MeasureSeriesVoltage8bit(chcfg, chgain, &ch1vdc16, chcfg, chgain, &ch2vdc16, true);
		eeprom_write_byte((uint8_t *)&adc_offset8CH1[calibration_ADCOffsetCell(chcfg, chgain)], -ch1vdc16 >> 8);
		eeprom_write_byte((uint8_t *)&adc_offset8CH2[calibration_ADCOffsetCell(chcfg, chgain)], -ch2vdc16 >> 8);
		if ((ch1vdc16 > 16383) || (ch1vdc16 < -16383)) setbit(result, CALIBRATION_ERROR_OutOfRange);
		if ((ch2vdc16 > 16383) || (ch2vdc16 < -16383)) setbit(result, CALIBRATION_ERROR_OutOfRange);
	}
	return result;
}