/*
 * data.c part of XMEGA Oscilloscope
 *
 * Created: 2016-12-16
 * Atmel Studio 6 (Version: 6.2.1563 - Service Pack 2)
 *  Author: Krzysztof Markiewicz
 *  www.obbo.pl
 *
 * This program is distributed under the terms of the GNU General Public License
 */ 

#include <util/delay.h>
#include "data.h"
#include "DAC.h"
#include "mathasm.h"
#include "bitlib.h"


#define VERSION_HARDWARE		"11"
#define VERISON_SOTWARE			"12"

const char DATA_DEVICE_INFO[DATA_DEVICE_INFO_LENGTH]	PROGMEM = "OscyloskopX." VERSION_HARDWARE "." VERISON_SOTWARE " (Build time: " __DATE__ " " __TIME__ ")";
uint8_t EEMEM data_custom[USB_ENDPOINT1_AUXDATA_LENGTH] = {0};


uint16_t data_AllChannelDataLength(uint8_t chlength, uint8_t chcfg);
uint16_t data_AWGDataLength(uint8_t daclength, uint8_t daccfg);
uint8_t data_DACBufferMaxBlocks(uint8_t sizech, uint8_t daccfg, uint8_t chcfg);
uint8_t data_CHBufferMaxBlocks(uint8_t sizedac, uint8_t daccfg, uint8_t chcfg);
uint8_t data_ChannelControlDefault(void);




void data_MainBufferInit(uint8_t sizedac, uint8_t sizech, uint8_t daccfg, uint8_t chcfg) {
	uint16_t startpool = 0, logicpool;
	
	// DAC
	if (testbit(daccfg, MAIN_CHANNEL_CONTROL_Enable)) {
		awgbuffers.wave = (uint16_t *)(mainbuffer);
		awgbuffers.dac = (uint16_t *)(mainbuffer + (DAC_DATA_BYTE_SIZE * sizedac * DATA_BUFFER_BLOCK_SIZE));
		awgbuffers.out = (uint16_t *)(mainbuffer + 2 * (DAC_DATA_BYTE_SIZE * sizedac * DATA_BUFFER_BLOCK_SIZE));
		startpool += 3 * (DAC_DATA_BYTE_SIZE * sizedac * DATA_BUFFER_BLOCK_SIZE);
	} else {
		awgbuffers.wave = (uint16_t *)(mainbuffer);
		awgbuffers.dac = (uint16_t *)(mainbuffer);
		awgbuffers.out = (uint16_t *)(mainbuffer);
		startpool = 0;
	}
	logicpool = startpool;
	// USB
	usbbuffers.all = (uint8_t *)(mainbuffer + startpool);
	usbbuffers.config = (frameconfig_t *)(mainbuffer + startpool);
	startpool += FRAME_HEADER_LENGTH;
	if (testbit(chcfg, MAIN_CHANNEL_CH1)) {
		usbbuffers.ch1 = (uint8_t *)(mainbuffer + startpool);
		startpool += (sizech * DATA_BUFFER_BLOCK_SIZE);
	}
	if (testbit(chcfg, MAIN_CHANNEL_CH2)) {
		usbbuffers.ch2 = (uint8_t *)(mainbuffer + startpool); 
		startpool += (sizech * DATA_BUFFER_BLOCK_SIZE);
	}
	if (testbit(chcfg, MAIN_CHANNEL_CHD)) {
		usbbuffers.chd = (uint8_t *)(mainbuffer + startpool);
		startpool += (sizech * DATA_BUFFER_BLOCK_SIZE);
	}
	usbbuffers.mmch1 = (channela_t *)(mainbuffer + startpool);
	startpool += DATA_CHMETER_RESULT_LENGTH;
	usbbuffers.mmch2 = (channela_t *)(mainbuffer + startpool);
	startpool += DATA_CHMETER_RESULT_LENGTH;
	usbbuffers.mmchd = (channeld_t *)(mainbuffer + startpool);
	startpool += DATA_CHDCONTROL_LENGTH;
	// MSO
	if (testbit(chcfg, MAIN_CHANNEL_CH1)) {
		databuffers.ch0 = (uint8_t *)(mainbuffer + startpool);
		startpool += (sizech * DATA_BUFFER_BLOCK_SIZE + ADC_BUFFER_EXTEND);
	}
	if (testbit(chcfg, MAIN_CHANNEL_CH2)) {
		databuffers.ch1 = (uint8_t *)(mainbuffer + startpool);
		startpool += (sizech * DATA_BUFFER_BLOCK_SIZE + ADC_BUFFER_EXTEND);
	}
	if (testbit(chcfg, MAIN_CHANNEL_CHD)) {
		databuffers.chd = (uint8_t *)(mainbuffer + startpool);
		startpool += (sizech * DATA_BUFFER_BLOCK_SIZE + ADC_BUFFER_EXTEND);
	}
	// LOGIC
	sniffbuffers.all = (uint8_t *)(mainbuffer + logicpool);
	sniffbuffers.config = (sniffconfig_t *)(mainbuffer + logicpool);
	logicpool += SNIFFER_CONFIG_LENGTH;
	sniffbuffers.rx = (uint8_t *)(mainbuffer + logicpool);
	sniffbuffers.tx = (uint8_t *)(mainbuffer + logicpool + SNIFFER_USART_BUFFER_LENGTH);
	sniffbuffers.dx = (uint8_t *)(mainbuffer + logicpool + 2 * SNIFFER_USART_BUFFER_LENGTH);
	sniffbuffers.i2c = (uint8_t *)(mainbuffer + logicpool);
	sniffbuffers.i2cack = (uint8_t *)(mainbuffer + logicpool + SNIFFER_I2C_BUFFER_LENGTH);
	sniffbuffers.mosi = (uint8_t *)(mainbuffer + logicpool);
	sniffbuffers.miso = (uint8_t *)(mainbuffer + logicpool + SNIFFER_SPI_BUFFER_LENGTH);
}

void data_BufferClear8(uint8_t *buffer, uint16_t length, uint8_t value) {
	uint16_t i;
	
	for (i = 0; i < length; i++) {
		*buffer++ = value;
	}
}

void data_BufferClear16(uint16_t *buffer, uint16_t length, uint16_t value) {
	uint16_t i;
	
	for (i = 0; i < length; i++) {
		*buffer++ = value;
	}
}

uint16_t data_FrameLength(uint8_t sizech, uint8_t chcfg) {
	uint16_t result = 0;
	
	if (testbit(chcfg, MAIN_CHANNEL_CH1)) result += sizech;
	if (testbit(chcfg, MAIN_CHANNEL_CH2)) result += sizech;
	if (testbit(chcfg, MAIN_CHANNEL_CHD)) result += sizech;
	result = FRAME_HEADER_LENGTH + result * DATA_BUFFER_BLOCK_SIZE + 2 * DATA_CHMETER_RESULT_LENGTH + DATA_CHDCONTROL_LENGTH;
	return result;
}

uint16_t data_SnifferDataLength(void) {
	return (SNIFFER_BUFFER_LENGTH + SNIFFER_CONFIG_LENGTH);
}

uint16_t data_ChannelDataLength(uint8_t sizech) {
	return (sizech * DATA_BUFFER_BLOCK_SIZE + ADC_BUFFER_EXTEND);
}

uint16_t data_AllChannelDataLength(uint8_t chlength, uint8_t chcfg) {
	uint16_t result = 0;
	
	if (testbit(chcfg, MAIN_CHANNEL_CH1)) result += data_ChannelDataLength(chlength);
	if (testbit(chcfg, MAIN_CHANNEL_CH2)) result += data_ChannelDataLength(chlength);
	if (testbit(chcfg, MAIN_CHANNEL_CHD)) result += data_ChannelDataLength(chlength);
	return result;
}

uint16_t data_AWGDataLength(uint8_t daclength, uint8_t daccfg) {
	uint16_t result = 0;
	
	if (testbit(daccfg, MAIN_CHANNEL_CONTROL_Enable)) result += 3 * daclength;
	result = result * DAC_DATA_BYTE_SIZE * DATA_BUFFER_BLOCK_SIZE;
	return result;
}

uint8_t data_DACBufferMaxBlocks(uint8_t sizech, uint8_t daccfg, uint8_t chcfg) {
	uint8_t result = 0;
	uint16_t length = 0;
	
	length += data_FrameLength(sizech, chcfg);
	length += data_AllChannelDataLength(sizech, chcfg);
	while ((length + data_AWGDataLength(result, daccfg)) <= DATA_MAIN_BUFFER_MAX_SIZE) {
		result++;
		if (result > DAC_BUFFER_BLOCKS_MAX) break;
	}
	if (result > 0)	{
		result = result - 1;
	}
	return result;
}

uint8_t data_CHBufferMaxBlocks(uint8_t sizedac, uint8_t daccfg, uint8_t chcfg) {
	uint8_t result = 0;
	uint16_t length = 0;
	
	length += data_AWGDataLength(sizedac, daccfg);
	while ((length + data_FrameLength(result, chcfg) + data_AllChannelDataLength(result, chcfg)) <= DATA_MAIN_BUFFER_MAX_SIZE) {
		result++;
		if (result > CH_BUFFER_BLOCKS_MAX) break;
	}
	if (result > 0)	{
		result = result - 1;
	}
	return result;
}

uint8_t data_ChannelEnabled(uint8_t ch1ctrl, uint8_t ch2ctrl, uint8_t chdctrl) {
	uint8_t result = 0;

	if (testbit(ch1ctrl, MAIN_CHANNEL_CONTROL_Enable)) setbit(result, MAIN_CHANNEL_CH1);
	if (testbit(ch2ctrl, MAIN_CHANNEL_CONTROL_Enable)) setbit(result, MAIN_CHANNEL_CH2);
	if (testbit(chdctrl, MAIN_CHANNEL_CONTROL_Enable)) setbit(result, MAIN_CHANNEL_CHD);
	return result;
} 

uint8_t ReadCalibrationByte(uint8_t location) {
	uint8_t result;

	/* Load the NVM Command register to read the calibration row. */
	NVM_CMD = NVM_CMD_READ_CALIB_ROW_gc;
	result = pgm_read_byte(location);
	/* Clean up NVM Command register. */
	NVM_CMD = NVM_CMD_NO_OPERATION_gc;
	return result;
}

void data_WriteCalibration(const uint16_t idx, const uint32_t value) {
	switch(idx) {
		case CALIBRATION_DAC_Offset:
		case CALIBRATION_DAC_Gain:
			dac_Calibrate((uint8_t)idx, (uint8_t)value);
			break;
		case CALIBRATION_ADC_Clear:
			calibration_ADCOffsetClear(ADC_OFFSET_TAB_LENGTH, ADC_OFFSET_TAB_LENGTH);
			setbit(GPIO_Request, MAIN_REQUEST_MSO_Update);
			break;
		case CALIBRATION_ADC_Auto:
			calibration_ADCOffset();
			setbit(GPIO_Request, MAIN_REQUEST_MSO_Update);
			setbit(GPIO_Request, MAIN_REQUEST_Update);
			setbit(GPIO_Action, MSO_TRIGGER_STATUS_Break);
			break;
	}
}

void data_WriteControl(const uint16_t idx, const uint32_t value) {
    switch (idx) {
	    case MAIN_CONTROL_COUNTER_Start:
			setbit(GPIO_MeterFrequency, METER_FREQUENCY_Start);
			break;
		case MAIN_CONTROL_COUNTER_Stop:
			setbit(GPIO_MeterFrequency, METER_FREQUENCY_Stop);
			break;
		case MAIN_CONTROL_SCOPE_Start:
			clrbit(GPIO_Action, MSO_TRIGGER_STATUS_Stop);
			clrbit(GPIO_Action, MSO_TRIGGER_STATUS_Triggered);
			break;
		case MAIN_CONTROL_SCOPE_Stop:
			setbit(GPIO_Action, MSO_TRIGGER_STATUS_Stop);
			clrbit(GPIO_Action, MSO_TRIGGER_STATUS_Triggered);
			setbit(GPIO_Request, MAIN_REQUEST_Update);
			break;
		case MAIN_CONTROL_TRIGGER_Force:
			setbit(GPIO_Action, MSO_TRIGGER_STATUS_Force);
			clrbit(GPIO_Action, MSO_TRIGGER_STATUS_Stop);
			setbit(GPIO_Request, MAIN_REQUEST_Update);
			break;
		case MAIN_CONTORL_SNIFFER_Start:
			setbit(GPIO_Action, SNIFFER_REQUEST_Run);
			break;
		case MAIN_CONTORL_SNIFFER_Stop:
			clrbit(GPIO_Action, SNIFFER_REQUEST_Run);
			break;
		case MAIN_CONTORL_SNIFFER_Clear:
			setbit(GPIO_Action, SNIFFER_REQUEST_Clear);
			break;
	}
}

uint8_t data_WriteRecordToConfigurationConfirm(volatile uint8_t *result, const uint16_t idx, const uint32_t value) {
	uint8_t tmp, length = 1;
	 
	*result = MAIN_CONFIGURATION_RESULT_Accepted;
	setbit(GPIO_Request, MAIN_REQUEST_Update);
    switch (idx) {
		case MAIN_CONFIGURATION_SamplingRate:
			master.samplerate = (uint8_t)value;
			#if PORTE_PIN7_TEST_CLOCK_ENABLE == 1
				if (master.samplerate > SAMPLING_RATE_62k5ksps) {
					master.samplerate = SAMPLING_RATE_62k5ksps;
					*result = MAIN_CONFIGURATION_RESULT_Truncated;
				}
			#endif
			if (master.samplerate >= SAMPLING_RATE_RangeOut) {
				master.samplerate = SAMPLING_RATE_RangeOut - 1;
				*result = MAIN_CONFIGURATION_RESULT_Truncated;
			}
			clrbit(GPIO_Action, MSO_TRIGGER_STATUS_Triggered);
			setbit(GPIO_Action, MSO_TRIGGER_STATUS_Break);
			break;
		case MAIN_CONFIGURATION_CH1ctrl:
			length = 2;
			tmp = data_CHBufferMaxBlocks(master.dacbufferblocks, GPIO_DACctrl, data_ChannelEnabled((uint8_t)value, GPIO_CH2ctrl, GPIO_CHDctrl));
			if (master.chbufferblocks > tmp) {
				result[0] = MAIN_CONFIGURATION_RESULT_NoEnoughMemory;
				result[1] = tmp;
				break;
			}
			if (testbit(GPIO_CH1ctrl, MAIN_CHANNEL_CONTROL_Enable) != testbit((uint8_t)value, MAIN_CHANNEL_CONTROL_Enable)) {
				setbit(GPIO_Request, MAIN_REQUEST_BufferResize);
			}
			GPIO_CH1ctrl = (uint8_t)value;
			setbit(GPIO_Request, MAIN_REQUEST_MSO_Update);
			break;
		case MAIN_CONFIGURATION_CH2ctrl:
			length = 2;
			tmp = data_CHBufferMaxBlocks(master.dacbufferblocks, GPIO_DACctrl, data_ChannelEnabled(GPIO_CH1ctrl, (uint8_t)value, GPIO_CHDctrl));
			if (master.chbufferblocks > tmp) {
				result[0] = MAIN_CONFIGURATION_RESULT_NoEnoughMemory;
				result[1] = tmp;
				break;
			}
			if (testbit(GPIO_CH2ctrl, MAIN_CHANNEL_CONTROL_Enable) != testbit((uint8_t)value, MAIN_CHANNEL_CONTROL_Enable)) {
				setbit(GPIO_Request, MAIN_REQUEST_BufferResize);
			}
			GPIO_CH2ctrl = (uint8_t)value;
			setbit(GPIO_Request, MAIN_REQUEST_MSO_Update);
			break;
		case MAIN_CONFIGURATION_CHDctrl:
			length = 2;
			tmp = data_CHBufferMaxBlocks(master.dacbufferblocks, GPIO_DACctrl, data_ChannelEnabled(GPIO_CH1ctrl, GPIO_CH2ctrl, (uint8_t)value));
			if (master.chbufferblocks > tmp) {
				result[0] = MAIN_CONFIGURATION_RESULT_NoEnoughMemory;
				result[1] = tmp;
				break;
			}
			if (testbit(GPIO_CHDctrl, MAIN_CHANNEL_CONTROL_Enable) != testbit((uint8_t)value, MAIN_CHANNEL_CONTROL_Enable)) {
				setbit(GPIO_Request, MAIN_REQUEST_BufferResize);
			}
			GPIO_CHDctrl = (uint8_t)value;
			setbit(GPIO_Request, MAIN_REQUEST_MSO_Update);
			break;
		case MAIN_CONFIGURATION_CHDmask:
			GPIO_CHDmask = (uint8_t)value;
			setbit(GPIO_Request, MAIN_REQUEST_MSO_Update);
			break;
		case MAIN_CONFIGURATION_Trigger:
			GPIO_Trigger = (uint8_t)value;   
			setbit(GPIO_Action, MSO_TRIGGER_STATUS_Break);
			break;
		case MAIN_CONFIGURATION_Mixed:
			if (testbit(GPIO_Mixed, MAIN_MSO_MODE_Roll) != testbit((uint8_t)value, MAIN_MSO_MODE_Roll)) {
				clrbit(GPIO_Action, MSO_TRIGGER_STATUS_Triggered);
				setbit(GPIO_Action, MSO_TRIGGER_STATUS_Break);
			}
			GPIO_Mixed = (uint8_t)value;
			break;
		case MAIN_CONFIGURATION_Mode:
			master.mainmode = (uint8_t)value;
			setbit(GPIO_Request, MAIN_REQUEST_MSO_Update);
			setbit(GPIO_Request, MAIN_REQUEST_MM_Clear);
			setbit(GPIO_Action, MSO_TRIGGER_STATUS_Break);
			clrbit(GPIO_Action, MSO_TRIGGER_STATUS_Triggered);
			clrbit(GPIO_Status, MAIN_STATUS_Acquired);
			clrbit(GPIO_Action, SNIFFER_REQUEST_Run);
			break;
		case MAIN_CONFIGURATION_DACctrl:
			length = 2;
			tmp = data_DACBufferMaxBlocks(master.chbufferblocks, (uint8_t)value, data_ChannelEnabled(GPIO_CH1ctrl, GPIO_CH2ctrl, GPIO_CHDctrl));
			if (master.dacbufferblocks > tmp) {
				result[0] = MAIN_CONFIGURATION_RESULT_NoEnoughMemory;
				result[1] = data_CHBufferMaxBlocks(master.dacbufferblocks, (uint8_t)value, data_ChannelEnabled(GPIO_CH1ctrl, GPIO_CH2ctrl, GPIO_CHDctrl));
				break;
			}
			GPIO_DACctrl = (uint8_t)value;
			setbit(GPIO_Request, MAIN_REQUEST_BufferResize);
			setbit(GPIO_Request, MAIN_REQUEST_DAC_Update);
			break;
		case MAIN_CONFIGURATION_MeterFrequency:
			GPIO_MeterFrequency = (uint8_t)value;
			setbit(GPIO_Request, MAIN_REQUEST_MM_Clear);
			break;
		case MAIN_CONFIGURATION_SniffOption:
			GPIO_SniffOption = (uint8_t)value;
			setbit(GPIO_Request, MAIN_REQUEST_MSO_Update);
			break;
		case MAIN_CONFIGURATION_CH1gain:
			master.ch1gain = (uint8_t)value;
			if (master.ch1gain >= MAIN_CHANNEL_GAIN_RangeOut) {
				master.ch1gain = MAIN_CHANNEL_GAIN_RangeOut - 1;
				*result = MAIN_CONFIGURATION_RESULT_Truncated;
			}
			setbit(GPIO_Action, MSO_TRIGGER_STATUS_Break);
			setbit(GPIO_Request, MAIN_REQUEST_MSO_Update);
			break;
		case MAIN_CONFIGURATION_CH2gain:
			master.ch2gain = (uint8_t)value;
			if (master.ch2gain >= MAIN_CHANNEL_GAIN_RangeOut) {
				master.ch2gain = MAIN_CHANNEL_GAIN_RangeOut - 1;
				*result = MAIN_CONFIGURATION_RESULT_Truncated;
			}
			setbit(GPIO_Action, MSO_TRIGGER_STATUS_Break);
			setbit(GPIO_Request, MAIN_REQUEST_MSO_Update);
			break;
		case MAIN_CONFIGURATION_TriggerHold:
			master.triggerhold = (uint8_t)value;
			break;
		case MAIN_CONFIGURATION_TriggerPos:
			master.triggerpos = (uint16_t)value;
			setbit(GPIO_Action, MSO_TRIGGER_STATUS_Break);
			break;
		case MAIN_CONFIGURATION_TriggerSource:
			GPIO_TriggerSource = (uint8_t)value;
			if (GPIO_TriggerSource > MAIN_TRIGGER_SOURCE_RangeOut) {
				GPIO_TriggerSource = MAIN_TRIGGER_SOURCE_RangeOut - 1;
				*result = MAIN_CONFIGURATION_RESULT_Truncated;
			}
			setbit(GPIO_Action, MSO_TRIGGER_STATUS_Break);
			setbit(GPIO_Request, MAIN_REQUEST_MM_Clear);
			break;
		case MAIN_CONFIGURATION_GateSource:
			GPIO_GateSource = (uint8_t)value;
			if (GPIO_GateSource > MAIN_TRIGGER_SOURCE_RangeOut) {
				GPIO_GateSource = MAIN_TRIGGER_SOURCE_RangeOut - 1;
				*result = MAIN_CONFIGURATION_RESULT_Truncated;
			}
			setbit(GPIO_Action, MSO_TRIGGER_STATUS_Break);
			setbit(GPIO_Request, MAIN_REQUEST_MM_Clear);
			break;
		case MAIN_CONFIGURATION_TriggerLevel:
			master.triggerlevel = (int8_t)value;
			if (master.triggerlevel > TRIGGRT_LEVEL_VALUE_MAX) master.triggerlevel = TRIGGRT_LEVEL_VALUE_MAX;
			if (master.triggerlevel < -TRIGGRT_LEVEL_VALUE_MAX) master.triggerlevel = -TRIGGRT_LEVEL_VALUE_MAX;
			setbit(GPIO_Action, MSO_TRIGGER_STATUS_Break);
			break;
		case MAIN_CONFIGURATION_Window1:
			master.window1 = (uint8_t)value;
			if (master.window1 > TRIGGRT_WINDOW_VALUE_MAX) master.window1 = TRIGGRT_WINDOW_VALUE_MAX;
			if (master.window1 < -TRIGGRT_WINDOW_VALUE_MAX) master.window1 = -TRIGGRT_WINDOW_VALUE_MAX;
			setbit(GPIO_Action, MSO_TRIGGER_STATUS_Break);
			break;
		case MAIN_CONFIGURATION_Window2:
			master.window2 = (uint8_t)value;
			if (master.window2 > TRIGGRT_WINDOW_VALUE_MAX) master.window2 = TRIGGRT_WINDOW_VALUE_MAX;
			if (master.window2 < -TRIGGRT_WINDOW_VALUE_MAX) master.window2 = -TRIGGRT_WINDOW_VALUE_MAX;
			setbit(GPIO_Action, MSO_TRIGGER_STATUS_Break);
			break;
		case MAIN_CONFIGURATION_AutoTrigger:
			master.autotrigger = (uint8_t)value;
			setbit(GPIO_Action, MSO_TRIGGER_STATUS_Break);
			break;
		case MAIN_CONFIGURATION_SniffProtocol:
			master.sniffprotocol = (uint8_t)value;
			if (master.sniffprotocol > SNIFFER_PROTOCOL_RangeOut) {
				master.sniffprotocol = SNIFFER_PROTOCOL_RangeOut - 1;
				*result = MAIN_CONFIGURATION_RESULT_Truncated;
			}
			clrbit(GPIO_Action, SNIFFER_REQUEST_Run);
			break;
		case MAIN_CONFIGURATION_USARTBitRate:
			master.bitrate = (uint8_t)value;
			break;
		case MAIN_CONFIGURATION_SniffStrobeEdge:	
			master.sniffportedge = (uint8_t)value;
			break;
		case MAIN_CONFIGURATION_DACBufferBlocks:
			length = 2;
			tmp = data_DACBufferMaxBlocks(master.chbufferblocks, GPIO_DACctrl, data_ChannelEnabled(GPIO_CH1ctrl, GPIO_CH2ctrl, GPIO_CHDctrl));
			if ((uint8_t)value > tmp) {
				result[0] = MAIN_CONFIGURATION_RESULT_NoEnoughMemory;
				result[1] = tmp;
				break;
			}
			master.dacbufferblocks = (uint8_t)value;
			if (master.dacbufferblocks < DAC_BUFFER_BLOCKS_MIN) {
				master.dacbufferblocks = DAC_BUFFER_BLOCKS_MIN;
				*result = MAIN_CONFIGURATION_RESULT_Truncated;
			} else if (master.dacbufferblocks > DAC_BUFFER_BLOCKS_MAX) {
				master.dacbufferblocks = DAC_BUFFER_BLOCKS_MAX;
				*result = MAIN_CONFIGURATION_RESULT_Truncated;
			}			
			setbit(GPIO_Request, MAIN_REQUEST_BufferResize);
			break;
		case MAIN_CONFIGURATION_CHBufferBlocks:
			length = 2;
			tmp = data_CHBufferMaxBlocks(master.dacbufferblocks, GPIO_DACctrl, data_ChannelEnabled(GPIO_CH1ctrl, GPIO_CH2ctrl, GPIO_CHDctrl));
			if ((uint8_t)value > tmp) {
				result[0] = MAIN_CONFIGURATION_RESULT_NoEnoughMemory;
				result[1] = tmp;
				break;
			}
			master.chbufferblocks = (uint8_t)value;
			if (master.chbufferblocks < CH_BUFFER_BLOCKS_MIN) {
				master.chbufferblocks = CH_BUFFER_BLOCKS_MIN;
				*result = MAIN_CONFIGURATION_RESULT_Truncated;
			} else if (master.chbufferblocks > CH_BUFFER_BLOCKS_MAX) {
				master.chbufferblocks = CH_BUFFER_BLOCKS_MAX;
				*result = MAIN_CONFIGURATION_RESULT_Truncated;
			}
			setbit(GPIO_Request, MAIN_REQUEST_BufferResize);
			break;
		case MAIN_CONFIGURATION_SweepType:
			master.sweep.type = (uint8_t)value;
			setbit(GPIO_Request, MAIN_REQUEST_AWG_BufferUpdate);
			break;
		case MAIN_CONFIGURATION_SweepStart:
			master.sweep.start = (uint8_t)value;
			break;
		case MAIN_CONFIGURATION_SweepEnd:
			master.sweep.end = (uint8_t)value;
			break;
		case MAIN_CONFIGURATION_SweepSpeed:
			master.sweep.speed = (uint8_t)value;
			if (master.sweep.speed > AWG_SWEEP_SPEED_MAX) {
				master.sweep.speed = AWG_SWEEP_SPEED_MAX;
				*result = MAIN_CONFIGURATION_RESULT_Truncated;
			}
			break;
	    case MAIN_CONFIGURATION_WaveAmplitude:
			master.wave.amplitude = (uint8_t)value;
			setbit(GPIO_Request, MAIN_REQUEST_AWG_BufferUpdate);
			break;
	    case MAIN_CONFIGURATION_WaveType:
			master.wave.type = (uint8_t)value;
			setbit(GPIO_Request, MAIN_REQUEST_AWG_Update);
			setbit(GPIO_Request, MAIN_REQUEST_AWG_BufferUpdate);
			break;
	    case MAIN_CONFIGURATION_WaveDuty:
			master.wave.duty = (uint8_t)value;
		    if (master.wave.duty < AWG_WAVE_MIN_Duty) {
				master.wave.duty = AWG_WAVE_MIN_Duty;
				*result = MAIN_CONFIGURATION_RESULT_Truncated;
			}
			setbit(GPIO_Request, MAIN_REQUEST_AWG_BufferUpdate);
			break;
	    case MAIN_CONFIGURATION_WaveOffset:
			master.wave.offset = (int8_t)value;
			setbit(GPIO_Request, MAIN_REQUEST_AWG_BufferUpdate);
			break;
	    case MAIN_CONFIGURATION_WaveFrequency:
			master.wave.frequency = (uint32_t)value;
			if (master.wave.frequency < AWG_WAVE_MIN_Frequency) {
				master.wave.frequency = AWG_WAVE_MIN_Frequency;
				*result = MAIN_CONFIGURATION_RESULT_Truncated;
			} else if (master.wave.frequency > AWG_WAVE_MAX_Frequency) {
				master.wave.frequency = AWG_WAVE_MAX_Frequency;
				*result = MAIN_CONFIGURATION_RESULT_Truncated;
			}
			setbit(GPIO_Request, MAIN_REQUEST_AWG_BufferUpdate);
			break; 
	    case MAIN_CONFIGURATION_LevelConstant:
			master.wave.constvalue = (int16_t)value;
			setbit(GPIO_Request, MAIN_REQUEST_AWG_Update);
			setbit(GPIO_Request, MAIN_REQUEST_AWG_BufferUpdate);
			break;
	}
	return length;
}

void data_WriteRecordToConfiguration(const uint16_t idx, const uint32_t value) {
	uint8_t dummy;
	
	data_WriteRecordToConfigurationConfirm(&dummy, idx, value);
}

void data_WriteBulkToConfiguration(uint8_t count) {
	uint8_t i;
	
	for (i = 0; i < count; i++) {
		data_WriteRecordToConfiguration(i, *((uint32_t *)(awgbuffers.wave) + i));
	}
}

void data_WriteDataBulk(const uint16_t idx) {
	switch(idx) {
		case MAIN_BULK_DATA_AWG_WaveFB:
		    eeprom_write_block((uint8_t *)awgbuffers.wave, (uint8_t *)(awg_wavecustom), USB_ENDPOINT1_AUXDATA_LENGTH);
			break;
		case MAIN_BULK_DATA_AWG_WaveLB:
			eeprom_write_block((uint8_t *)awgbuffers.wave, (uint8_t *)(awg_wavecustom) + USB_ENDPOINT1_AUXDATA_LENGTH, USB_ENDPOINT1_AUXDATA_LENGTH);
			break;
		case MAIN_BULK_DATA_Config:
			data_WriteBulkToConfiguration(MAIN_CONFIGURATION_RangeOut);
			setbit(GPIO_Status, MAIN_STATUS_USB_Connected);
			break;
		case MAIN_BULK_DATA_Custom:
			eeprom_write_block((uint8_t *)awgbuffers.wave, (uint8_t *)(data_custom), USB_ENDPOINT1_AUXDATA_LENGTH);
			break;
	}
}

uint8_t data_RequestDataTransfer(volatile uint8_t *buffer, const uint16_t idx, const uint16_t value) {
	uint8_t i, length = 0;
	uint8_t *address = (uint8_t *)(DATA_DEVICE_INFO);
	
	switch (idx) {
		case MAIN_DATA_REQUEST_FirmwareVersion:
			for (i = 0; i < DATA_DEVICE_INFO_LENGTH; i++) {
				*buffer++ = pgm_read_byte(address++);
			}
			length = DATA_DEVICE_INFO_LENGTH;
			break;
		case MAIN_DATA_REQUEST_Measure:
			for (i = 0; i < DATA_CHMETER_RESULT_LENGTH; i++) {
				*buffer++ = *((uint8_t *)usbbuffers.mmch1 + i);
			}
			for (i = 0; i < DATA_CHMETER_RESULT_LENGTH; i++) {
				*buffer++ = *((uint8_t *)usbbuffers.mmch2 + i);
			}
			length = (2 * DATA_CHMETER_RESULT_LENGTH);
			break;
		case MAIN_DATA_REQUEST_DeviceStatus:
			*buffer = GPIO_Status;
			length = 1;
			break;
		case MAIN_DATA_REQUEST_DACBufferMax:
			*buffer = data_DACBufferMaxBlocks(master.chbufferblocks, GPIO_DACctrl, data_ChannelEnabled(GPIO_CH1ctrl, GPIO_CH2ctrl, GPIO_CHDctrl));
			length = 1;
			break;
		case MAIN_DATA_REQUEST_CHBufferMax:
			*buffer = data_CHBufferMaxBlocks(master.dacbufferblocks, GPIO_DACctrl, data_ChannelEnabled(GPIO_CH1ctrl, GPIO_CH2ctrl, GPIO_CHDctrl));
			length = 1;
			break;
		case MAIN_DATA_REQUEST_ActualFrequency:
			*(uint32_t *)buffer = awg_OutputFrequency(master.dacbufferblocks * DATA_BUFFER_BLOCK_SIZE, master.wave.frequency);
			length = 4;
			break;
		case MAIN_DATA_REQUEST_CustomData:
			*buffer = (uint8_t)(eeprom_read_byte(&data_custom[value]));
			length = 1;
			break;
	}
	return length;
}

void data_LoadDefaultConfiguration(void) {
	data_WriteRecordToConfiguration(MAIN_CONFIGURATION_DACBufferBlocks, DAC_BUFFER_BLOCKS_DEFAULT);
	data_WriteRecordToConfiguration(MAIN_CONFIGURATION_CHBufferBlocks, CH_BUFFER_BLOCKS_DEFAULT);
	data_WriteRecordToConfiguration(MAIN_CONFIGURATION_WaveAmplitude, AWG_WAVE_DEF_Amplitude);
	data_WriteRecordToConfiguration(MAIN_CONFIGURATION_WaveType, AWG_WAVE_DEF_Type);
	data_WriteRecordToConfiguration(MAIN_CONFIGURATION_WaveDuty, AWG_WAVE_DEF_Duty);
	data_WriteRecordToConfiguration(MAIN_CONFIGURATION_WaveOffset, AWG_WAVE_DEF_Offset);
	data_WriteRecordToConfiguration(MAIN_CONFIGURATION_WaveFrequency, AWG_WAVE_DEF_Frequency);
	data_WriteRecordToConfiguration(MAIN_CONFIGURATION_LevelConstant, AWG_WAVE_DEF_ConstValue);
	data_WriteRecordToConfiguration(MAIN_CONFIGURATION_CH1ctrl, data_ChannelControlDefault());
	data_WriteRecordToConfiguration(MAIN_CONFIGURATION_CH2ctrl, data_ChannelControlDefault());
	data_WriteRecordToConfiguration(MAIN_CONFIGURATION_CH1gain, MAIN_CHANNEL_GAIN_24V);
	data_WriteRecordToConfiguration(MAIN_CONFIGURATION_CH2gain, MAIN_CHANNEL_GAIN_24V);
}

uint8_t data_ChannelControlDefault(void) {
	uint8_t result = 0;
	
	setbit(result, MAIN_CHANNEL_CONTROL_Enable);
	setbit(result, MAIN_CHANNEL_CONTROL_FilterDC);
	return result;
}

int8_t data_ReadCHDataOffset(uint8_t channel, uint8_t chcfg, uint8_t chgain) {
	if (channel == MAIN_CHANNEL_CH1) {
		return eeprom_read_byte((uint8_t *)&adc_offset8CH1[calibration_ADCOffsetCell(chcfg, chgain)]);
	} else if (channel == MAIN_CHANNEL_CH2) {
		return eeprom_read_byte((uint8_t *)&adc_offset8CH2[calibration_ADCOffsetCell(chcfg, chgain)]);
	}
	return 0;
}

void data_StoreCHDataGain(uint8_t ch1gain, uint8_t ch2gain) {
 	usbbuffers.mmch1->gain = ch1gain;
 	usbbuffers.mmch2->gain = ch2gain;
}

uint8_t data_ChannelAnalog(void) {
	uint8_t result = 0;
	
	result += (1 << MAIN_CHANNEL_CH1);
	result += (1 << MAIN_CHANNEL_CH2);
	return result;
}

