/*
 * input.c part of XMEGA Oscilloscope
 *
 * Created: 2016-12-18 20:34:00
 * Atmel Studio 6 (Version: 6.2.1563 - Service Pack 2)
 *  Author: Krzysztof Markiewicz
 *  www.obbo.pl
 *
 * This program is distributed under the terms of the GNU General Public License
 */ 

#include "input.h"
#include "data.h"


void input_ChannelGain(uint8_t gain, uint8_t *adcgain, bool *chdiv10);


void input_CHxDCFilter(PORT_t *port, const bool state) {
	if (state) {
		port->OUT &= ~(PRK_ACDC_bm);
	} else {
		port->OUT |= PRK_ACDC_bm;
	}
}

void input_DCFilter(const bool state) {
	input_CHxDCFilter(&CH1_PORT, state);
	input_CHxDCFilter(&CH2_PORT, state);
}

void input_CHxDiv10(PORT_t *port, const bool state) {
	if (state) {
		port->OUT &= ~(PRK_DIV10_bm);
	} else {
		port->OUT |= PRK_DIV10_bm;
	}
}

void input_Div10(const bool state) {
	input_CHxDiv10(&CH1_PORT, state);
	input_CHxDiv10(&CH2_PORT, state);
}

void input_AnalogPortConfig(const uint8_t ch1cfg, const uint8_t ch1gain, const uint8_t ch2cfg, const uint8_t ch2gain) {
	input_CHxDCFilter(&CH1_PORT, testbit(ch1cfg, MAIN_CHANNEL_CONTROL_FilterDC));
	input_CHxDCFilter(&CH2_PORT, testbit(ch2cfg, MAIN_CHANNEL_CONTROL_FilterDC));
	input_CHxDiv10(&CH1_PORT, input_ChannelDiv10(ch1gain));
	input_CHxDiv10(&CH2_PORT, input_ChannelDiv10(ch2gain));
}

void input_LogicPortConfig(PORT_t *port, bool invert, uint8_t edge) {
	uint8_t i;

	for (i = 0; i < 8; i++) {
		input_LogicPinConfig(port, i, invert, edge);
	}
}

void input_LogicPinConfig(PORT_t *port, uint8_t pin, bool invert, uint8_t edge) {
	uint8_t cfg;
	
	if (pin < 8) {
		cfg = *(&(port->PIN0CTRL) + pin);
		cfg &= ~(PORT_ISC_gm);
		if (invert) {
			cfg |= PORT_INVEN_bm;
		} else {
			cfg &= ~(PORT_INVEN_bm);
		}
		*(&(port->PIN0CTRL) + pin) = cfg | edge;
	}
}

uint8_t input_ChannelADCGain(uint8_t gain) {
	uint8_t adcgain;
	bool chdiv10;
	
	input_ChannelGain(gain, &adcgain, &chdiv10);
	return adcgain;
}

bool input_ChannelDiv10(uint8_t gain) {
	uint8_t adcgain;
	bool chdiv10;
	
	input_ChannelGain(gain, &adcgain, &chdiv10);
	return chdiv10;
}

void input_ChannelGain(uint8_t gain, uint8_t *adcgain, bool *chdiv10) {
	if (gain >= MAIN_CHANNEL_GAIN_RangeOut) gain = MAIN_CHANNEL_GAIN_RangeOut - 1;
	if (gain >= MAIN_CHANNEL_GAIN_2400mV) {
		*adcgain = gain - MAIN_CHANNEL_GAIN_2400mV;
		*chdiv10 = false;
	} else {
		*adcgain = gain;
		*chdiv10 = true;
	}
}
