/*
 * input.h part of XMEGA Oscilloscope
 *
 * Created: 2016-12-18 20:34:13
 * Atmel Studio 6 (Version: 6.2.1563 - Service Pack 2)
 *  Author: Krzysztof Markiewicz
 *  www.obbo.pl
 *
 * This program is distributed under the terms of the GNU General Public License
 */ 

#ifndef INPUT_H_
#define INPUT_H_

#include <stdbool.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include "bitlib.h"


// PORT DEFINITIONS
#define LED_PORT			VPORT3
#define LED_GRN				4
#define LED_RED				5

#define LED_GRN_ON()		setbit(VPORT3.OUT, LED_GRN)
#define LED_GRN_OFF()		clrbit(VPORT3.OUT, LED_GRN)
#define LED_RED_ON()		setbit(VPORT3.OUT, LED_RED)
#define LED_RED_OFF()		clrbit(VPORT3.OUT, LED_RED)

#define CH1_PORT			PORTA
#define CH2_PORT			PORTB
#define CHD_PORT			PORTC

#define INPUT_SCHMITT_PIN	0
#define INPUT_EXT_PIN		1

#define PRK_DIV10_bp		3
#define PRK_DIV10_bm		(1 << PRK_DIV10_bp)
#define PRK_ACDC_bp			4
#define PRK_ACDC_bm			(1 << PRK_ACDC_bp)


enum input_portedges {
	INPUT_PORT_SENSE_EDGE_Bothedges,
	INPUT_PORT_SENSE_EDGE_Rising,
	INPUT_PORT_SENSE_EDGE_Falling,
	INPUT_PORT_SENSE_Level
};

void input_CHxDCFilter(PORT_t *port, const bool state);
void input_DCFilter(const bool state);
void input_CHxDiv10(PORT_t *port, const bool state);
void input_Div10(const bool state);
void input_AnalogPortConfig(const uint8_t ch1cfg, const uint8_t ch1gain, const uint8_t ch2cfg, const uint8_t ch2gain);
void input_LogicPortConfig(PORT_t *port, bool invert, uint8_t edge);
void input_LogicPinConfig(PORT_t *port, uint8_t pin, bool invert, uint8_t edge);
uint8_t input_ChannelADCGain(uint8_t gain);
bool input_ChannelDiv10(uint8_t gain);

#endif /* INPUT_H_ */