/*
 * multimeter.h part of XMEGA Oscilloscope
 *
 * Created: 2016-12-16 09:47:14
 * Atmel Studio 6 (Version: 6.2.1563 - Service Pack 2)
 *  Author: Krzysztof Markiewicz
 *  www.obbo.pl
 *
 * This program is distributed under the terms of the GNU General Public License
 */ 

#ifndef MULTIMETER_H_
#define MULTIMETER_H_

#include <stdint.h>
#include <stdbool.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include "data.h"


enum mm_error {
	MM_ERROR_ADC_ConversionTimeout,
	MM_ERROR_MEASURE_OutOfRange
};


void mm_MeasureClearResult(channela_t *channel);
void mm_MeasureSeriesVoltage12bit(const uint8_t ch1cfg, const uint8_t ch1gain, int32_t *ch1vdc, const uint8_t ch2cfg, const uint8_t ch2gain, int32_t *ch2vdc, bool calibrate);
void mm_MeasureVDC(const uint8_t ch1cfg, const uint8_t ch1gain, channela_t *ch1vdc, const uint8_t ch2cfg, const uint8_t ch2gain, channela_t *ch2vdc);
void mm_MeasureSeriesVoltage8bit(const uint8_t ch1cfg, const uint8_t ch1gain, int16_t *ch1vdc, const uint8_t ch2cfg, const uint8_t ch2gain, int16_t *ch2vdc, bool calibrate);
void mm_MeasureVAC(int8_t *buffer, const uint16_t length, channela_t *channel);
void mm_MeasureFrequency(config_t *measuremode, const uint8_t trigersource, channela_t *result, bool fastmode);
uint8_t mm_MeasurePeriod(config_t *measuremode, const uint8_t trigersource, channela_t *result, channela_t *duty);
uint8_t mm_CountPulses(config_t *measuremode, const uint8_t trigersource, channela_t *result);
uint8_t mm_CountPulsesWithGate(config_t *measuremode, const uint8_t trigersource, const uint8_t gate, channela_t *result, uint8_t gatemode);


#endif /* MULTIMETER_H_ */